
#ifndef TSimplePropertySet_CPP
#define TSimplePropertySet_CPP

#include "TSimplePropertySet.hpp"

templateClassImp(TSimplePropertySet);

template <class N, class V>
TSimplePropertySet<N,V>::TSimplePropertySet()
{ // Default constructor

	nam = N();
	sl=list<TProperty<N,V> >();
}

template <class N, class V>
TSimplePropertySet<N,V>::TSimplePropertySet(const N& name)
{ // Named property set

	nam = name;
	sl=list<TProperty<N,V> >();
}

template <class N, class V>
TSimplePropertySet<N,V>::TSimplePropertySet(const TSimplePropertySet<N,V>& source)
{ // Copy constructor
	
	nam = source.nam;

	sl = source.sl;
}


template <class N, class V>
TSimplePropertySet<N,V>::~TSimplePropertySet()
{ // Destructor


}

// Iterator functions
template <class N, class V>
TSimplePropertySet<N,V>::iterator TSimplePropertySet<N,V>::Begin()
{ // Return iterator at begin of composite

	return sl.begin();
}

template <class N, class V>
TSimplePropertySet<N,V>::const_iterator TSimplePropertySet<N,V>::Begin() const
{ // Return const iterator at begin of composite

	return sl.begin();
}

template <class N, class V>
TSimplePropertySet<N,V>::iterator TSimplePropertySet<N,V>::End()
{ // Return iterator after end of composite

	return sl.end();
}

template <class N, class V>
TSimplePropertySet<N,V>::const_iterator TSimplePropertySet<N,V>::End() const
{ // Return const iterator after end of composite

	return sl.end();
}

// Selectors
template <class N, class V>
int TSimplePropertySet<N,V>::Count() const
{ // Return the number of functions in the composite

	return sl.size();
}

template <class N, class V>
	N TSimplePropertySet<N,V>::operator ()() const
{ // The name of the property set

		return nam;
}

template <class N, class V>
	bool TSimplePropertySet<N,V>::hasProperty(const N& search_name) const
{ // Is the property in the list?

	// We iterate over the list until we find the value
	const_iterator it;

	for (it=sl.begin(); it!=sl.end(); it++)
	{ 

		if ((*it).name() == search_name)
		{
			return true;
		}
	}

	return false;
}


// Modifiers
template <class N, class V>
	void TSimplePropertySet<N,V>::operator () (const N& name)
{

		nam = name;
}


template <class N, class V>
	TProperty<N,V> TSimplePropertySet<N,V>::value(const N& name) const
{


	// We iterate over the list until we find the value
	const_iterator it;

	for (it=sl.begin(); it!=sl.end(); it++)
	{ 

		if ((*it).name() == name)
		{
			return (*it);
		}
	}

	// If not found, then take no action, caveat emptor	

}

template <class N, class V>
void TSimplePropertySet<N,V>::set(const N& name, const V & value)
{
	iterator it;

	for (it=sl.begin(); it!=sl.end(); it++)
	{ 
		if ((*it).name() == name)
		{
			(*it)(value);
			break;
		}
	}
}

// Add functions
template <class N, class V>
void TSimplePropertySet<N,V>::add(const TProperty<N,V>& scf)
{ // Add a property to the list


	sl.push_back(scf);
}

template <class N, class V>
void TSimplePropertySet<N,V>::add(const TSimplePropertySet<N,V>& scf)
{ // Add a property set to the list


	//	sl.merge(scf.sl); This function does not work because scf is emptied 
	// by merge() and thus cannot be const.

	// We iterate over the SECOND list until we find the value
	const_iterator it;

	for (it=scf.sl.begin(); it!=scf.sl.end(); it++)
	{ 

		add((*it));
	}

}

template <class N, class V>
	void TSimplePropertySet<N,V>::remove(const N& value)
{ // Remove all elements with 'value' O(N)

	// We iterate over the list until we find the value
	iterator it;

	for (it=sl.begin(); it!=sl.end(); it++)
	{ 

		if ((*it)() == value)
		{
			erase(it);
		}
	}

}



// Operators
template <class N, class V>
	TSimplePropertySet<N,V>& TSimplePropertySet<N,V>::operator = (const TSimplePropertySet<N,V>& source)
{ // Assignment

	// Exit if same object
	if (this==&source) return *this;


	nam = source.nam;

	sl = source.sl;

	return *this;
}


#endif