
#ifndef TSimplePropertySet_hpp
#define TSimplePropertySet_hpp

#include "TProperty.cpp"
#include <list>
#include <set>

using namespace std;

//template <class N, class V> class EntityVisitor;


template <class N, class V> class TSimplePropertySet
{
private:

	N nam;		// The name of the set

	// The SimplePropertySet list using the STL list
	list<TProperty<N,V> > sl;


public:
	// User can use the STL iterator
	typedef typename list<TProperty<N,V> >::iterator iterator;
	typedef typename list<TProperty<N,V> >::const_iterator const_iterator;

	// Constructors and destructor
	TSimplePropertySet();		// Default constructor
	TSimplePropertySet(const N& name);		// Named property set
	TSimplePropertySet(const TSimplePropertySet<N,V>& source);	// Copy constructor

	virtual ~TSimplePropertySet();	// Destructor

	// Iterator functions
    iterator Begin();			// Return iterator at begin of composite
	const_iterator Begin() const;// Return const iterator at begin of composite
	iterator End();				// Return iterator after end of composite
    const_iterator End() const;	// Return const iterator after end of composite

	// Selectors
	int Count() const;				// The number of properties in the list
	N operator ()() const;			// The name of the property set
	bool hasProperty(const N& search_name) const;	// Is the property in the list?
	TProperty<N,V> value(const N& name) const;	// Get the Property based on a name

	// Modifiers
	void operator () (const N& name);			// Change the name of the property set
	void set(const N& name, const V & value);

	// Add and remove functions (sort of mixin or embedded inheritance)
	void add(const TProperty<N,V>& p); 
	void add(const TSimplePropertySet<N,V>& p); 
	void remove(const N& value);		// Remove all elements with 'value' O(N)

	
	// Operators
	TSimplePropertySet<N,V>& operator = (const TSimplePropertySet<N,V>& source);
        ClassDef(TSimplePropertySet,1);

};

#endif	// SimplePropertySet_hpp
